/*
 * Spar - a text frontend for the Spar library 
 * Copyright (C) 2000,2001 Davide Angelocola <davide178@inwind.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include <string.h>
#include <spar/sl_application.h>

#include "commands.h"
#include "commands_table.h"

extern void spar_exit (int __code);

COMMAND (cmd_exit)
{
  sl_application_exit (SL_SUCCESS);

}

COMMAND (cmd_warranty)
{
  sl_warranty ();
}

COMMAND (cmd_set_old_value)
{
  if (argv[1] == NULL)
    sl_error_throw (SL_ERROR_CLASS_ERROR, SL_ERROR_CUSTOM,
		    "%s command gets at least one arguments.", argv[0]);
  sl_variable_old (argv[1]);
}

extern struct cmd_entry Commands[];

COMMAND (cmd_help)
{
  if (argc == 1)
    {
      print_cmd_table ();
    }
  else
    {
      int index = check_command (argv[1]);

      sl_writeln ("Command: '%-s'", Commands[index].cmd);
      sl_writeln ("\tsyntax    : %-s %-s",
		  Commands[index].cmd, Commands[index].usage);
      sl_writeln ("\twhat does : %-s", Commands[index].help);
    }
}

COMMAND (cmd_add_var_comment)
{
  if (argc < 3)
    {
      sl_error_throw (SL_ERROR_CLASS_ERROR, SL_ERROR_CUSTOM,
		      "this command gets at least 2 arguments. \n"
		      "Type 'help %s' for the command's syntax.", argv[0]);
      return;
    }

  else
    {
      sl_message (comment) = " ";
      register int i, comment_length = 0;
      for (i = 2; i < argc; i++)
	comment_length = strlen (argv[i]) + 1;

      if (comment_length > SL_MESSAGE_LENGHT)
	{
	  sl_error_throw (SL_ERROR_CLASS_ERROR, SL_ERROR_MESSAGE, "");
	  return;
	}

      for (i = 2; i < argc; i++)
	{
	  strcat (comment, argv[i]);
	  strcat (comment, " ");
	}

      if ((sl_variable_comment (argv[1], comment)))
	{
	  sl_error_throw (SL_ERROR_CLASS_ERROR, SL_ERROR_MESSAGE,
			  "failed to comment the variable '%s'.", argv[1]);
	}
    }
}

COMMAND (cmd_consts)
{
  sl_constant_print_all ();
}

COMMAND (cmd_vars)
{
  sl_variable_print_all ();
}

COMMAND (cmd_funcs)
{
  switch (argc)
    {
    case 1:
      {
	sl_function_print_all ();
	break;
      }

    case 2:
      {
	sl_function_help doc;
	if (sl_function_make_help (argv[1], &doc))
	  {
	    sl_writeln ("Function \"%s\":", doc.name);
	    sl_writeln ("\t[arguments] :\t%u", doc.args);
	    sl_writeln ("\t[math form] :\t%s", doc.math);
	    sl_writeln ("\t[math type] :\t%s", doc.type);
	  }
	else
	  sl_error_throw (SL_ERROR_CLASS_ERROR, SL_ERROR_MESSAGE,
			  "unknown function '%s'.", argv[1]);
	break;
      }

    default:
      break;
    }
  return;
}

/*
 * The following commands implements the module support of this program.
 * There are 3 basic actions that you can do with modules:
 *     running      run a module   
 *     showing      show infos about a module
 */
COMMAND (cmd_mod_run)
{
  if (argv[1] == NULL)
    {
      sl_writeln
	("No module specified. Please pick a module from this list.");
      sl_module_manager_print_all ();
    }
  else
    sl_module_manager_run (argv[1]);
}

COMMAND (cmd_mod_show)
{
  switch (argc)
    {
    case 1:
      sl_module_manager_print_all ();
      break;

    case 2:
      sl_module_manager_print (argv[1]);
      break;

    default:
      break;
    }
}
