/*
 * The Spar Library - modular math parser
 * Copyright (C) 2000,2001 Davide Angelocola <davide178@inwind.it>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *
 */

#include <spar/sl_complex.h>
#include <spar/sl_math_library.h>

sl_complex
sl_complex_sin (sl_complex z)
{
  double s = sl_sin (SL_COMPLEX_RE (z));
  double c = sl_cos (SL_COMPLEX_RE (z));
  double u = sl_exp (SL_COMPLEX_IM (z));
  double v = 1. / u;

  // TODO: use the ldexp function
  u = (u + v) * sl_pow (2., -1.);
  v = u - v;
  SL_COMPLEX_RE (z) = u * s;
  SL_COMPLEX_IM (z) = c * v;

  return z;
}

sl_complex
sl_complex_cos (sl_complex z)
{
  double s = sl_sin (SL_COMPLEX_RE (z));
  double c = sl_cos (SL_COMPLEX_RE (z));
  double u = sl_exp (SL_COMPLEX_IM (z));
  double v = 1. / u;

  u = (u + v) * sl_pow (2., -1.);
  v = u - v;
  SL_COMPLEX_RE (z) = c * u;
  SL_COMPLEX_IM (z) = -s * v;

  return z;
}

sl_complex
sl_complex_tan (sl_complex z)
{
  double s = sl_sin (SL_COMPLEX_RE (z));
  double c = sl_cos (SL_COMPLEX_RE (z));
  double u = sl_exp (SL_COMPLEX_IM (z));
  double v = 1. / u;
  double d;

  u = (u + v) * sl_pow (2., -1.);
  v = u - v;
  d = sl_pow_2 (c) + sl_pow_2 (v);
  SL_COMPLEX_RE (z) = s * c / d;
  SL_COMPLEX_IM (z) = u * v / d;

  return z;
}
