/*
 * The Spar Library -
 * Copyright (C) 2000,2001 Davide Angelocola <davide178@inwind.it>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *
 */  
  
/* 
 *  This random number generator is simple, fast and portable. 
 *  I have run this routine's output through a number of tests for 
 *  randomness and it does very well. The least significant bits 
 *  of the return value should be discarded. 
 */ 
#include <spar/sl_random.h>
  
sl_size seed1 = 0x5324879f;

sl_size seed2 = 0xb78d0945;


sl_size 
 sl_rand (void) 
{
  
sl_size sum;
  

    /* 
     *  Add the two seeds together 
     */ 
    sum = seed1 + seed2;
  

    /* 
     *  If the add generated a carry, increment
     *  the result of the addition.
     */ 
    if (sum < seed1 || sum < seed2)
    sum++;
  

    /* 
     *  Seed2 becomes old seed1, seed1 becomes result 
     */ 
    seed2 = seed1;
  
seed1 = sum;
  

    /* 
     *  Return the result of the addition 
     */ 
    return sum;

}



/* 
 *  This routine sets the seed to a different
 *  starting value for each run of a program.
 *  It requires the ftime() function and
 *  struct timeb to be defined.  They can be
 *  found in timeb.h.
 */ 
#include <sys/timeb.h>

void 
sl_randomize (void) 
{
  
struct timeb t;
  

ftime (&t);
  
seed1 = (sl_size) (t.time + t.millitm);

}


